"""
Common filenames and locations for elevation, meshes and outputs.
This script is the heart of all scripts in the folder
"""

import sys
import os
from os.path import join, exists
from time import localtime, strftime, gmtime
from anuga.utilities.polygon import read_polygon, number_mesh_triangles
from anuga.utilities.system_tools import get_user_name, get_host_name


#-----
# Get system data - don't fiddle with this.
# These values can be used to distinguish between two similar runs by two
# different users or runs by the same user on two different machines.
#-----

user = get_user_name()
host = get_host_name()

#-------------------------------------------------------------------------------
# Directory setup
#-------------------------------------------------------------------------------

# this section needs to be updated to reflect the modelled community.
# Note, the user needs to set up the directory system accordingly
state = 'western_australia'
scenario_name = 'busselton'
scenario_folder = 'busselton_tsunami_scenario'

#-------------------------------------------------------------------------------
# Initial Conditions
#-------------------------------------------------------------------------------

# Model specific parameters.
# One or all can be changed each time the run_model script is executed
tide = 0                # ??
event_number = 27283    # the event number
alpha = 0.1             # smoothing parameter for mesh
friction=0.01           # manning's friction coefficient
starttime=0             # start time for simulation
finaltime=80000         # final time for simulation

setup = 'final'  # Final can be replaced with trial or basic.
                 # Either will result in a coarser mesh that will
                 # allow a faster, but less accurate, simulation.

#-------------------------------------------------------------------------------
# Output Filename
#-------------------------------------------------------------------------------

# Important to distinguish each run - ensure str(user) is included!
# Note, the user is free to include as many parameters as desired
# This should be moved to run_model.py, with the user providing the list to join
# for example: output_comment_list = [setup, str(tide), str(event_number), str(user)]
output_comment = '_'.join([setup, str(tide), str(event_number), str(user)])

#-------------------------------------------------------------------------------
# Input Data
#-------------------------------------------------------------------------------

# ELEVATION DATA
# Used in build_elevation.py
# Format for ascii grids, as produced in ArcGIS + a projection file
ascii_grid_filenames = ['busselton_v2',   # Topo
                        'grid_250m_clip'] # Busselton Topo

# Format for point is x,y,elevation (with header)
point_filenames = ['Busselton_Contour0.txt',    # Coastline
                   'Busselton_BeachSurvey.txt', # Beach survey
                   'Busselton_NavyFinal.txt',   # Bathymetry
                   'Busselton_Chart.txt',       # Bathymetry Charts
                   'Busselton_Digitised.txt',   # Digitised Fairsheet
                   'Busselton_250m.txt',        # 250m
                   'Bunbury_TIN.txt',           # Bunbury aoi TIN'd in ArcGIS
                   'Busselton_TIN.txt',         # Busselton aoi TIN'd in ArcGIS
                   'XYAHD_clip.txt']            # To extend boundary

# BOUNDING POLYGON - for data clipping and estimate of triangles in mesh
# Used in build_elevation.py
# Format for points easting,northing (no header)
bounding_polygon_filename = 'bounding_polygon.csv'
bounding_polygon_maxarea = 100000

# INTERIOR REGIONS -  for designing the mesh
# Used in run_model.py
# Format for points easting,northing (no header)                    
interior_regions_data = [['busselton_1km.csv', 500],
                         ['bunbury_1km.csv', 500],
                         ['busselton_2km.csv', 10000],
                         ['bunbury_2km.csv', 10000],
                         ['island1.csv', 10000],
                         ['island2.csv', 10000],
                         ['coast_5km_d20m.csv', 40000]]

# LAND - used to set the initial stage/water to be offcoast only
# Used in run_model.py.  Format for points easting,northing (no header)
land_initial_conditions_filename = [['initial_condition_extend.csv', 0],
                                    ['initial_condition_marina.csv', 0]]

# GAUGES - for creating timeseries at a specific point
# Used in get_timeseries.py.  
# Format easting,northing,name,elevation (with header)
gauges_filename = 'gauges.csv'

# BUILDINGS EXPOSURE - for identifying inundated houses
# Used in run_building_inundation.py
# Format latitude,longitude etc (geographic)
building_exposure_filename = 'busselton_res_clip.csv' #from NEXIS

# BOUNDING POLYGON - used in build_boundary.py and run_model.py respectively
# NOTE: when files are put together the points must be in sequence
# For ease go clockwise!
# Check the run_model.py for boundary_tags

# Thinned ordering file from Hazard Map (geographic)
# Format is index,latitude,longitude (with header)
urs_order_filename = 'thinned_boundary_ordering_extend.csv'

# Landward bounding points
# Format easting,northing (no header)
landward_boundary_filename = 'landward_boundary_extend.csv'

# MUX input filename
# Must exist in $MUXHOME/mux directory.
# File is prodiced by EventSelection.
# If MUX file not used, do "mux_input_filename = None".
mux_input_filename = r'C:\temp\event_026633.list'

#-------------------------------------------------------------------------------
# Clipping regions for export to asc and regions for clipping data
# Final inundation maps should only be created in regions of the finest mesh
#-------------------------------------------------------------------------------

# ASCII export grid for Busselton
xminBusselton = 340000
xmaxBusselton = 352000
yminBusselton = 6271500
ymaxBusselton = 6280000

# ASCII export grid for Bunbury
xminBunbury = 369000
xmaxBunbury = 381000
yminBunbury = 6308000
ymaxBunbury = 6316500


################################################################################
################################################################################
####             NOTE: NOTHING SHOULD CHANGE BELOW THIS POINT.              ####
################################################################################
################################################################################

#-------------------------------------------------------------------------------
# Sanity checks
#-------------------------------------------------------------------------------

# Test that environment variables are defined.
# A little late, but better here than up in user-changeable data.

# Environment variable names.
# The inundation directory, not the data directory.
ENV_INUNDATIONHOME = 'INUNDATIONHOME'

# Path to MUX data
ENV_MUXHOME = 'MUXHOME'

### Check we have required environment variables set
##sanity_error = False               # checked at bottom of this file
##
##if os.getenv(ENV_INUNDATIONHOME) is None:
##    print "Environment variable '%s' is not set" % ENV_INUNDATIONHOME
##    sanity_error = True
##
##if os.getenv(ENV_MUXHOME) is None:
##    print "Environment variable '%s' is not set" % ENV_MUXHOME
##    sanity_error = True
##
### check for errors detected so far
##if sanity_error:
##    print 'You must fix the above errors before continuing.'
##    sys.exit(10)

#-------------------------------------------------------------------------------
# Output Elevation Data
#-------------------------------------------------------------------------------

# Output filename for elevation
# this is a combination of all the data generated in build_elevation.py
combined_elevation_basename = scenario_name + '_combined_elevation'

#-------------------------------------------------------------------------------
# Directory Structure
#-------------------------------------------------------------------------------

# determines time for setting up output directories
time = strftime('%Y%m%d_%H%M%S', localtime()) 
gtime = strftime('%Y%m%d_%H%M%S', gmtime()) 
build_time = time + '_build'
run_time = time + '_run'

# create paths generated from environment variables.
home = join(os.getenv(ENV_INUNDATIONHOME), 'data') # Absolute path for data folder
##if not exists(home):
##    print "Sorry, data directory '%s' doesn't exist" % home
##    sanity_error = True
    
muxhome = os.getenv(ENV_MUXHOME)
##if not exists(muxhome):
##    print "Sorry, MUX directory '%s' doesn't exist" % muxhome
##    sanity_error = True
    
# check various directories/files that must exist
anuga_folder = join(home, state, scenario_folder, 'anuga')
##if not exists(anuga_folder):
##    print "Sorry, ANUGA directory '%s' doesn't exist" % anuga_folder
##    sanity_error = True

topographies_folder = join(anuga_folder, 'topographies')
##if not exists(topographies_folder):
##    print "Sorry, topo directory '%s' doesn't exist" % topographies_folder
##    sanity_error = True

polygons_folder = join(anuga_folder, 'polygons')
##if not exists(polygons_folder):
##    print "Sorry, polygon directory '%s' doesn't exist" % polygons_folder
##    sanity_error = True

boundaries_folder = join(anuga_folder, 'boundaries')
##if not exists(boundaries_folder):
##    print "Sorry, boundaries directory '%s' doesn't exist" % boundaries_folder
##    sanity_error = True

output_folder = join(anuga_folder, 'outputs')
##if not exists(output_folder):
##    print "Sorry, outputs directory '%s' doesn't exist" % output_folder
##    sanity_error = True

gauges_folder = join(anuga_folder, 'gauges')
##if not exists(gauges_folder):
##    print "Sorry, gauges directory '%s' doesn't exist" % gauges_folder
##    sanity_error = True

meshes_folder = join(anuga_folder, 'meshes')
##if not exists(meshes_folder):
##    print "Sorry, meshes directory '%s' doesn't exist" % meshes_folder
##    sanity_error = True

### if mux_input_filename has been defined, check if file exists
##if mux_input_filename:
##    if not exists(mux_input_filename):
##        print ("Sorry, MUX input file '%s' doesn't exist" % mux_input_filename)
##        sanity_error = True

#-----
# If these directories don't exist initially, create them.
# This should be moved to run_model.py.
#-----

event_folder = join(boundaries_folder, str(event_number))
if not exists(event_folder):
    print "Creating directory 'event_folder'=%s" % event_folder
    os.mkdir(event_folder)

#-------------------------------------------------------------------------------
# Location of input and output data
#-------------------------------------------------------------------------------

# The absolute pathname of the all elevation, generated in build_elevation.py
combined_elevation = join(topographies_folder, combined_elevation_basename)

# The absolute pathname of the mesh, generated in run_model.py
meshes = join(meshes_folder, scenario_name) + '.msh'

# The absolute pathname for the urs order points, used within build_boundary.py
urs_order = join(boundaries_folder, urs_order_filename)

# The absolute pathname for the landward points of the bounding polygon,
# Used within run_model.py)
landward_boundary = join(boundaries_folder, landward_boundary_filename)

# The absolute pathname for the .sts file, generated in build_boundary.py
event_sts = join(boundaries_folder, str(event_number), scenario_name)

# The absolute pathname for the output folder names
# Used for build_elevation.py
output_build = join(output_folder, build_time) + '_' + str(user) 
# Used for run_model.py
output_run = join(output_folder, run_time) + output_comment 
# Used by post processing
output_run_time = join(output_run, scenario_name) 

# The absolute pathname for the gauges file
# Used for get_timeseries.py
gauges = join(gauges_folder, gauges_filename)       

# The absolute pathname for the building file
# Used for run_building_inundation.py
building_exposure = join(gauges_folder, building_exposure_filename)

#-------------------------------------------------------------------------------
# Determine type of run
#-------------------------------------------------------------------------------

if setup == 'trial':
    print 'trial'
    scale_factor = 100
    time_thinning = 96
    yieldstep = 240
elif setup == 'basic': 
    print 'basic'
    scale_factor = 4
    time_thinning = 12
    yieldstep = 120
elif setup == 'final': 
    print 'final'
    scale_factor = 1
    time_thinning = 4
    yieldstep = 60
##else:
##    print ("Sorry, you must set the 'setup' variable to one of 'trial', "
##           "'basic' or 'final'. I found '%s'" % str(setup))
##    sanity_error = True

#-------------------------------------------------------------------------------
# Reading polygons and creating interior regions
#-------------------------------------------------------------------------------

# Create list of land polygons with initial conditions
land_initial_conditions = []
for filename, MSL in land_initial_conditions_filename:
    polygon = read_polygon(join(polygons_folder, filename))
    land_initial_conditions.append([polygon, MSL])

# Create list of interior polygons with scaling factor
interior_regions = []
for filename, maxarea in interior_regions_data:
    polygon = read_polygon(join(polygons_folder, filename))
    interior_regions.append([polygon, maxarea*scale_factor])

# Initial bounding polygon for data clipping 
bounding_polygon = read_polygon(join(polygons_folder,
                                     bounding_polygon_filename))
bounding_maxarea = bounding_polygon_maxarea*scale_factor

# Estimate the number of triangles                     
trigs_min = number_mesh_triangles(interior_regions,
                                  bounding_polygon, bounding_maxarea)

###-------------------------------------------------------------------------------
### Check for errors detected above.
###-------------------------------------------------------------------------------
##
##if sanity_error:
##    print 'You must fix the above errors before continuing.'
##    sys.exit(10)


print 'min estimated number of triangles', trigs_min
